@@ -134,7 +134,7 @@ Py::Object pysvn_client::cmd_ls( const Py::Tuple &a_args, const Py::Dict &a_kws
         entry_dict[ *py_name_name ] = Py::String( full_name, name_utf8 );
         entry_dict[ *py_name_kind ] = toEnumValue( dirent->kind );
         entry_dict[ *py_name_has_props ] = Py::Int( dirent->has_props );
-        entry_dict[ *py_name_size ] = Py::Long( Py::Float( double( static_cast<signed_int64>( dirent->size ) ) ) );
+        entry_dict[ *py_name_size ] = toFilesize( dirent->size );
         entry_dict[ *py_name_created_rev ] = Py::asObject( new pysvn_revision( svn_opt_revision_number, 0, dirent->created_rev ) );
         entry_dict[ *py_name_time ] = toObject( dirent->time );
         entry_dict[ *py_name_last_author ] = utf8_string_or_none( dirent->last_author );
@@ -237,7 +237,7 @@ extern "C" svn_error_t *list_receiver_c
         }
         if( baton->m_dirent_fields&SVN_DIRENT_SIZE )
         {
-            entry_dict[ *py_name_size ] = Py::Long( Py::Float( double( static_cast<signed_int64>( dirent->size ) ) ) );
+            entry_dict[ *py_name_size ] = toFilesize( dirent->size );
         }
         if( baton->m_dirent_fields&SVN_DIRENT_CREATED_REV )
         {
@@ -295,6 +295,9 @@ Py::Object pysvn_client::cmd_list( const Py::Tuple &a_args, const Py::Dict &a_kw
 #endif
 #if defined( PYSVN_HAS_CLIENT_LIST3 )
     { false, name_include_externals },
+#endif
+#if defined( PYSVN_HAS_CLIENT_LIST4 )
+    { false, name_patterns },
 #endif
     { false, NULL }
     };
@@ -331,6 +334,17 @@ Py::Object pysvn_client::cmd_list( const Py::Tuple &a_args, const Py::Dict &a_kw
 
     try
     {
+#if defined( PYSVN_HAS_CLIENT_LIST4 )
+        apr_array_header_t *patterns = NULL;
+        if( args.hasArg( name_patterns ) )
+        {
+            Py::Object py_patterns = args.getArg( name_patterns );
+            if( !py_patterns.isNone() )
+            {
+                patterns = arrayOfStringsFromListOfStrings( py_patterns, pool );
+            }
+        }
+#endif
         checkThreadPermission();
 
         PythonAllowThreads permission( m_context );
@@ -346,7 +360,23 @@ Py::Object pysvn_client::cmd_list( const Py::Tuple &a_args, const Py::Dict &a_kw
         list_baton.m_include_externals = include_externals;
 #endif
 
-#if defined( PYSVN_HAS_CLIENT_LIST3 )
+#if defined( PYSVN_HAS_CLIENT_LIST4 )
+        svn_error_t *error = svn_client_list4
+            (
+            norm_path.c_str(),
+            &peg_revision,
+            &revision,
+            patterns,
+            depth,
+            dirent_fields,
+            fetch_locks,
+            include_externals,
+            list_baton.callback(),
+            list_baton.baton(),
+            m_context,
+            pool
+            );
+#elif defined( PYSVN_HAS_CLIENT_LIST3 )
         svn_error_t *error = svn_client_list3
             (
             norm_path.c_str(),
