@@ -1026,7 +1026,13 @@ class MacOsxCompilerGCC(CompilerGCC):
         xcode_version_path = os.path.join( xcode_path, 'Contents', 'version.plist' )
         if os.path.exists( xcode_version_path ):
             import plistlib
-            xcode_version_info = plistlib.readPlist( '/Applications/Xcode.app/Contents/version.plist')
+            if sys.version_info.major == 2:
+                xcode_version_info = plistlib.readPlist( '/Applications/Xcode.app/Contents/version.plist' )
+
+            else:
+                with open( '/Applications/Xcode.app/Contents/version.plist', 'rb' ) as f:
+                    xcode_version_info = plistlib.load( f )
+
             self.xcode_version = [int(x) for x in xcode_version_info['CFBundleShortVersionString'].split( '.' )]
 
         else:
